@@ -25,8 +25,12 @@
 
 static GHashTable* calculation_functions_hash_table = NULL;
 static gchar** search_paths = { NULL };
+GQuark quark_animations_hash_table = 0;
 
 static void on_weak_notify(gpointer user_data, GObject* other_object);
+static GHashTable* get_animations_hash_table(GObject* object);
+static void setup_animation_valist(ClutterAnimation* animation, GObject* object, va_list args);
+static void setup_animation_array(ClutterAnimation* animation, GObject* object, gint n_properties, const gchar* const property_names[], const GValue* property_values);
 
 #define DEFINE_CALCULATION_FUNCTIONS(t) \
 	static gint compare_##t(const GValue* value, const GValue* comparable) { g##t v; g##t c; \
@@ -79,6 +83,8 @@ gboolean tangle_init(int* argc, char*** argv) {
 		tangle_value_register_calculation_functions(G_TYPE_FLOAT, &calculation_functions_float);
 		tangle_value_register_calculation_functions(G_TYPE_DOUBLE, &calculation_functions_double);
 
+		quark_animations_hash_table = g_quark_from_static_string("tangle-animations-hash-table");
+
 		success = TRUE;
 	}
 	
@@ -93,6 +99,14 @@ gpointer tangle_unref_object(gpointer object) {
 	return NULL;
 }
 
+ClutterActor* tangle_unparent_actor(ClutterActor* actor) {
+	if (actor) {
+		clutter_actor_unparent(actor);
+	}
+	
+	return NULL;
+}
+
 TanglePointers* tangle_pointers_new(gint n, ...) {
 	TanglePointers* pointers;
 	va_list args;
@@ -438,11 +452,13 @@ gchar* tangle_lookup_filename(const gchar* filename) {
 	if (g_path_is_absolute(filename)) {
 		path = g_strdup(filename);
 	} else {
-		for (csp = search_paths; !path && *csp; csp++) {
-			path = g_build_filename(*csp, filename, NULL);
-			if (!g_file_test(path, G_FILE_TEST_EXISTS)) {
-				g_free(path);
-				path = NULL;
+		if (search_paths) {
+			for (csp = search_paths; !path && *csp; csp++) {
+				path = g_build_filename(*csp, filename, NULL);
+				if (!g_file_test(path, G_FILE_TEST_EXISTS)) {
+					g_free(path);
+					path = NULL;
+				}
 			}
 		}
 		
@@ -469,6 +485,97 @@ void tangle_add_search_path(const gchar* search_path) {
 	search_paths[search_paths_length + 1] = NULL;
 }
 
+ClutterAnimation* tangle_object_animate(GObject* object, gulong mode, guint duration, ...) {
+	ClutterAnimation* animation;
+	va_list args;
+	
+	animation = clutter_animation_new();
+	clutter_animation_set_mode(animation, mode);
+	clutter_animation_set_duration(animation, duration);
+
+	va_start(args, duration);
+	setup_animation_valist(animation, object, args);
+	va_end(args);
+
+	return animation;
+}
+
+ClutterAnimation* tangle_object_animate_with_timeline(GObject* object, gulong mode, ClutterTimeline* timeline, ...) {
+	ClutterAnimation* animation;
+	va_list args;
+	
+	animation = clutter_animation_new();
+	clutter_animation_set_mode(animation, mode);
+	clutter_animation_set_timeline(animation, timeline);
+
+	va_start(args, timeline);
+	setup_animation_valist(animation, object, args);
+	va_end(args);
+
+	return animation;
+}
+
+ClutterAnimation* tangle_object_animatev(GObject* object, gulong mode, guint duration, gint n_properties, const gchar* const property_names[], const GValue* property_values) {
+	ClutterAnimation* animation;
+	
+	animation = clutter_animation_new();
+	clutter_animation_set_mode(animation, mode);
+	clutter_animation_set_duration(animation, duration);
+
+	setup_animation_array(animation, object, n_properties, property_names, property_values);
+
+	return animation;
+}
+
+ClutterAnimation* tangle_object_animate_valist(GObject* object, gulong mode, guint duration, va_list properties) {
+	ClutterAnimation* animation;
+	
+	animation = clutter_animation_new();
+	clutter_animation_set_mode(animation, mode);
+	clutter_animation_set_duration(animation, duration);
+
+	setup_animation_valist(animation, object, properties);
+
+	return animation;
+}
+	
+ClutterAnimation* tangle_object_animate_valist_with_timeline(GObject* object, gulong mode, ClutterTimeline* timeline, va_list properties) {
+	ClutterAnimation* animation;
+	va_list args;
+	
+	animation = clutter_animation_new();
+	clutter_animation_set_mode(animation, mode);
+	clutter_animation_set_timeline(animation, timeline);
+
+	setup_animation_valist(animation, object, properties);
+
+	return animation;
+}
+
+static gboolean is_value_user_data(gpointer key, gpointer value, gpointer user_data) {
+
+	return value == user_data;
+}
+
+gboolean tangle_object_stop_animation(GObject* object, ClutterAnimation* animation) {
+	gboolean retvalue = FALSE;
+	GHashTable* animations;
+	
+	animations = get_animations_hash_table(object);
+	
+	if (g_hash_table_foreach_remove(animations, is_value_user_data, animation)) {
+		g_object_unref(animation);
+
+		if (g_hash_table_size(animations) == 0) {
+			g_object_set_qdata(object, quark_animations_hash_table, NULL);
+		}
+
+		retvalue = TRUE;
+	}
+	
+	return retvalue;
+}
+
 static void on_weak_notify(gpointer user_data, GObject* other_object) {
 	TangleVault* vault;
 	GObject* object;
@@ -481,3 +588,108 @@ static void on_weak_notify(gpointer user_data, GObject* other_object) {
 	
 	tangle_vault_free(vault);
 }
+
+static GHashTable* get_animations_hash_table(GObject* object) {
+	GHashTable* animations;
+
+	if (!(animations = (GHashTable*)g_object_get_qdata(object, quark_animations_hash_table))) {
+		animations = g_hash_table_new_full(g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_object_unref);
+		g_object_set_qdata_full(object, quark_animations_hash_table, animations, (GDestroyNotify)g_hash_table_unref);
+	}
+
+	return animations;
+}
+
+static void on_animation_completed(ClutterAnimation* animation, gpointer user_data) {
+	GObject* object;
+	GHashTable* animations;
+
+	object = G_OBJECT(user_data);
+
+	animations = get_animations_hash_table(object);
+
+	g_hash_table_foreach_remove(animations, is_value_user_data, animation);
+	g_object_unref(animation);
+	
+	if (g_hash_table_size(animations) == 0) {
+		g_object_set_qdata(object, quark_animations_hash_table, NULL);
+	}
+}
+
+static void setup_animation_valist(ClutterAnimation* animation, GObject* object, va_list args) {
+	GObjectClass *klass;
+	GHashTable* animations;
+	const gchar* property_name;
+
+	clutter_animation_set_object(animation, object);
+	g_signal_connect(animation, "completed", G_CALLBACK(on_animation_completed), object);
+
+	klass = G_OBJECT_GET_CLASS(object);
+	animations = get_animations_hash_table(object);
+
+	while ((property_name = va_arg(args, gchar*))) {
+		GParamSpec *param_spec;
+		GValue value = { 0 };
+		gchar* error;
+		
+		if ((param_spec = g_object_class_find_property(klass, property_name))) {
+			g_value_init(&value, G_PARAM_SPEC_VALUE_TYPE(param_spec));
+			error = NULL;
+			G_VALUE_COLLECT(&value, args, 0, &error);
+			if (!error) {
+				ClutterAnimation* existing_animation;
+				
+				if ((existing_animation = g_hash_table_lookup(animations, property_name))) {
+					clutter_animation_unbind_property(existing_animation, property_name);
+					g_hash_table_remove(animations, property_name);
+				}
+				clutter_animation_bind(animation, property_name, &value);
+				g_hash_table_insert(animations, g_strdup(property_name), g_object_ref(animation)); // gobject_ref_sink does not work with 0.9.2!
+			} else {
+				g_warning("%s: %s", G_STRLOC, error);
+				g_free(error);
+				break;
+			}
+			g_value_unset(&value);
+		} else {
+			g_warning ("Cannot bind property '%s'.", property_name);
+			break;
+		}
+	}
+
+	clutter_timeline_start(clutter_animation_get_timeline(animation));
+}
+
+static void setup_animation_array(ClutterAnimation* animation, GObject* object, gint n_properties, const gchar* const property_names[], const GValue* property_values) {
+	GObjectClass *klass;
+	GHashTable* animations;
+	gint i;
+
+	clutter_animation_set_object(animation, object);
+	g_signal_connect(animation, "completed", G_CALLBACK(on_animation_completed), object);
+
+	klass = G_OBJECT_GET_CLASS(object);
+	animations = get_animations_hash_table(object);
+
+	for (i = 0; i < n_properties; i++) {
+		const gchar* property_name;
+		GParamSpec *param_spec;
+
+		property_name = property_names[i];	
+		if ((param_spec = g_object_class_find_property(klass, property_name))) {
+			ClutterAnimation* existing_animation;
+
+			if ((existing_animation = g_hash_table_lookup(animations, property_name))) {
+				clutter_animation_unbind_property(existing_animation, property_name);
+				g_hash_table_remove(animations, property_name);
+			}
+			clutter_animation_bind(animation, property_name, &property_values[i]);
+			g_hash_table_insert(animations, g_strdup(property_name), g_object_ref(animation)); // gobject_ref_sink does not work with 0.9.2!
+		} else {
+			g_warning ("Cannot bind property '%s'.", property_name);
+			break;
+		}
+	}
+
+	clutter_timeline_start(clutter_animation_get_timeline(animation));
+}
